"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.mockExternalComponents = mockExternalComponents;
exports.default = void 0;

var _path = _interopRequireDefault(require("path"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; var ownKeys = Object.keys(source); if (typeof Object.getOwnPropertySymbols === 'function') { ownKeys = ownKeys.concat(Object.getOwnPropertySymbols(source).filter(function (sym) { return Object.getOwnPropertyDescriptor(source, sym).enumerable; })); } ownKeys.forEach(function (key) { _defineProperty(target, key, source[key]); }); } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function mockExternalComponents(component, definition) {
  if (!definition) {
    throw new Error("Babel plugin of \"jest-mock-external-components\" is not enabled");
  }

  const {
    path: relativePathFromTest,
    identifier,
    type,
    fullPath: testPath
  } = definition;

  if (!testPath) {
    throw new Error("No test file path");
  }

  const fullPath = _path.default.join(_path.default.dirname(testPath), relativePathFromTest);

  const mockDefinitions = [];

  try {
    const main = require(fullPath);

    const mocks = main["Mocks"];

    if (!mocks) {
      return mockDefinitions;
    } // flat mocks by module path


    const flattenMocks = mocks.reduce((flattened, mock) => {
      const mockPath = mock.path;

      if (!flattened[mockPath]) {
        flattened[mockPath] = [];
      }

      flattened[mockPath].push({
        identifier: mock.identifier,
        type: mock.type
      });
      return flattened;
    }, {});

    for (const mockPath of Object.keys(flattenMocks)) {
      // path.dirname("..") and path.dirname("../") will result to "."
      // const mainDirPath = modulePath === "../" || modulePath === ".." ? modulePath : path.dirname(modulePath);
      const mainDirPath = _path.default.dirname(fullPath); // if mock is relative path then calculate properly path based on test path directory


      let newPath = mockPath.startsWith(".") ? _path.default.join(mainDirPath, mockPath) : mockPath;

      if (flattenMocks[mockPath] && flattenMocks[mockPath].length) {
        // const newPath = path.join(mainDirPath, mock.path);
        mockDefinitions.push({
          path: newPath,
          definition: () => {
            const actual = require.requireActual(newPath);

            let mockedModule = _objectSpread({}, actual);

            Object.defineProperty(mockedModule, "__esModule", {
              value: true
            });
            const mocks = flattenMocks[mockPath];

            for (const mock of mocks) {
              const mockIdentifiers = mock.identifier.split("."); // drop first identifier for namespace

              if (mock.type === "namespace") {
                mockIdentifiers.shift();
              } // import * as Comp from "./comp"; commonjs module.exports = ReactComp;


              if (!mockIdentifiers.length && mock.type === "namespace") {
                mockedModule = mock.identifier;
                delete mockedModule["__esModule"]; // Bail

                break;
              }

              if (mock.type === "default") {
                mockedModule["default"] = mock.identifier;
              } else {
                const mostRightIdentifier = mockIdentifiers.pop();

                if (!mostRightIdentifier) {
                  continue;
                }

                if (!mockIdentifiers.length) {
                  // top-level export
                  mockedModule[mostRightIdentifier] = mostRightIdentifier;
                } else {
                  // sub-level export, i.e. import * as E; E.A.B; -> E dropped, A is sublevel export and B is identifier name
                  // import { A }; A.B -> A is sublevel export
                  let mockedPath = mockedModule;

                  while (mockIdentifiers.length > 0) {
                    const subLevel = mockIdentifiers.shift();
                    mockedPath[subLevel] = _objectSpread({}, actual[subLevel]);
                    mockedPath = mockedPath[subLevel];
                  }

                  mockedPath[mostRightIdentifier] = mostRightIdentifier;
                }
              }
            }

            return mockedModule;
          }
        });
      }
    }

    jest.resetModules();
  } catch (_unused) {
    /* ignore */
  }

  return mockDefinitions;
}

var _default = mockExternalComponents;
exports.default = _default;
//# sourceMappingURL=index.js.map